# ╔══════════════════════════════════════════════════════════════════════════╗
# ║ Copyright 2022. quinn.7@foxmail.com All rights reserved.                 ║
# ║                                                                          ║
# ║ Licensed under the Apache License, Version 2.0 (the "License");          ║
# ║ you may not use this file except in compliance with the License.         ║
# ║ You may obtain a copy of the License at                                  ║
# ║                                                                          ║
# ║ http://www.apache.org/licenses/LICENSE-2.0                               ║
# ║                                                                          ║
# ║ Unless required by applicable law or agreed to in writing, software      ║
# ║ distributed under the License is distributed on an "AS IS" BASIS,        ║
# ║ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. ║
# ║ See the License for the specific language governing permissions and      ║
# ║ limitations under the License.                                           ║
# ╚══════════════════════════════════════════════════════════════════════════╝
"""
@ 模块     : 路径常量
@ 作者     : chenjiancheng
@ 邮箱     : quinn.7@foxmail.com
@ 编写时间 : 2022-08-10

@ 模块描述 :
    NULL

"""
from pathlib import Path


class DirPath: ...
class FilePath: ...


CWD = Path().cwd()
""" Path对象 :: 运行时路径 """


class DirPath:
    """
    [ 路径常量索引 :: 目录路径对象 ]

    ---
    描述: 
        NULL

    """

    Data = CWD.joinpath('Data')
    """ 数据目录 """

    Logs = Data.joinpath('Logs')
    """ 日志目录 """

    BaseCode = Data.joinpath('BaseCode')
    """ 底层编码目录 """


class FilePath:
    """
    [ 路径常量索引 :: 文件路径对象 ]

    ---
    描述: 
        NULL
        
    """

    Init = CWD.joinpath('AwakenEngineering.ini')

    Config = DirPath.Data.joinpath('AwakenConfig.yaml')
    """ 项目配置文件 """

    Database = DirPath.Data.joinpath('AwakenDatabase.db')
    """ 项目数据库文件 """


