# ╔══════════════════════════════════════════════════════════════════════════╗
# ║ Copyright 2022. quinn.7@foxmail.com All rights reserved.                 ║
# ║                                                                          ║
# ║ Licensed under the Apache License, Version 2.0 (the "License");          ║
# ║ you may not use this file except in compliance with the License.         ║
# ║ You may obtain a copy of the License at                                  ║
# ║                                                                          ║
# ║ http://www.apache.org/licenses/LICENSE-2.0                               ║
# ║                                                                          ║
# ║ Unless required by applicable law or agreed to in writing, software      ║
# ║ distributed under the License is distributed on an "AS IS" BASIS,        ║
# ║ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. ║
# ║ See the License for the specific language governing permissions and      ║
# ║ limitations under the License.                                           ║
# ╚══════════════════════════════════════════════════════════════════════════╝
"""
@ 模块     : 关键字索引
@ 作者     : chenjiancheng
@ 邮箱     : quinn.7@foxmail.com
@ 编写时间 : 2022-08-10

@ 模块描述 :
    NULL

"""
from . import keyword_api
from . import keyword_common
from . import keyword_script
from . import keyword_database


class AwakenKeyword: ...


class AwakenKeyword:
    """
    [ 关键字索引 ]

    ---
    描述:
        NULL

    """


    @property
    def Api(self) -> keyword_api:
        """
        [ API字段关键字 ]

        ---
        描述:
            NULL

        """
        return keyword_api


    @property
    def Common(self) -> keyword_common:
        """
        [ 通用字段关键字 ]

        ---
        描述:
            NULL

        """
        return keyword_common


    @property
    def Script(self) -> keyword_script:
        """
        [ 脚本字段关键字 ]

        ---
        描述:
            NULL

        """
        return keyword_script


    @property
    def DataBase(self) -> keyword_database:
        """
        [ 数据库关键字 ]

        ---
        描述:
            NULL

        """
        return keyword_database


KEYWORD: AwakenKeyword = AwakenKeyword()
""" 关键字索引实例 """
