# ╔══════════════════════════════════════════════════════════════════════════╗
# ║ Copyright 2022. quinn.7@foxmail.com All rights reserved.                 ║
# ║                                                                          ║
# ║ Licensed under the Apache License, Version 2.0 (the "License");          ║
# ║ you may not use this file except in compliance with the License.         ║
# ║ You may obtain a copy of the License at                                  ║
# ║                                                                          ║
# ║ http://www.apache.org/licenses/LICENSE-2.0                               ║
# ║                                                                          ║
# ║ Unless required by applicable law or agreed to in writing, software      ║
# ║ distributed under the License is distributed on an "AS IS" BASIS,        ║
# ║ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. ║
# ║ See the License for the specific language governing permissions and      ║
# ║ limitations under the License.                                           ║
# ╚══════════════════════════════════════════════════════════════════════════╝
"""
@ 模块     : 项目信息
@ 作者     : chenjiancheng
@ 邮箱     : quinn.7@foxmail.com
@ 编写时间 : 2022-08-10

@ 模块描述 :
    NULL

"""


class ProjectInfo: ...


class ProjectInfo:
    """
    [ 项目信息 ]

    """
    Name     = 'awaken2'
    Version  = '0.0.5'
    Author   = 'chenjiancheng'
    Email    = 'quinn.7@foxmail.com'
    Desc     = 'Awaken 是以 "关键字驱动" 为中心思想搭建的自动化测试框架。'
    Url      = 'https://gitee.com/catcat7/awaken2'
    Keywords = ['python', 'testing', 'automation', 'testautomation', 'awaken2']
